// Copyright (c) 2007  Anna Bigatti
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"
using namespace CoCoA;
using namespace std;

//----------------------------------------------------------------------
const string ShortDescription =
  "This example program shows how to create the rings Z and Q,       \n"
  "and how to use ring elements (RingElem).                          \n";

const string LongDescription =
  "Operations allowed on elements of the same ring:                  \n"
  "  zero(R) and one(R)                                              \n"
  "  a + b, a - b, a * b                                             \n"
  "  -a                                                              \n"
  "  a = b   (assignment)                                            \n"
  "  a == b  and  a != b  (comparison)                               \n"
  "  IsZero(a), IsOne(a), IsMinusOne(a)                              \n"
  "Moreover other operations might be allowed, for example:          \n"
  "  a > b   if the ring is ordered                                  \n"
  "  a / b   if the exact division is possible (and implemented!)    \n"
  "See ex-RingHom*.C for operations on elements of different rings.  \n";


//----------------------------------------------------------------------

void program()
{
  GlobalManager CoCoAFoundations;

  GlobalOutput() << ShortDescription << endl;

  ring Z = RingZ();
  ring Q = RingQ();

  GlobalOutput() << "A reminder: C++ prints out boolean values like this:" << endl
                 << " true   prints out as " << true << endl
                 << " false  prints out as " << false << endl
                 << endl;

  // When you create a RingElem, you must say to which ring it belongs.
  RingElem a(Z), b(Z);
  a = 1234;
  b = one(Z);

  GlobalOutput() << " a is " << a << "  and b is " << b << endl;

  GlobalOutput() << " a * (b + 3)    gives  " << (a * (b + 3)  ) << endl;
  GlobalOutput() << " a / 2          gives  " << (a / 2        ) << endl;
  // a/4 is not in Z, so it would throw a CoCoA Error
  GlobalOutput() << " IsZero(a)      gives  " << (IsZero(a)    ) << endl;
  GlobalOutput() << " a - b == 1233  gives  " << (a - b == 1233) << endl;
  GlobalOutput() << endl;

  RingElem p(Q), q(Q);
  p = 1234;
  q = RingElem(Q,3)/2;
  // We cannot write q = 3/2 because C++ reads it as an integer division!

  GlobalOutput() << " p is " << p << "  and q is " << q << endl;
  GlobalOutput() << " p / q          gives  " << (p / q        ) << endl;
  GlobalOutput() << " IsOne(q/q)     gives  " << (IsOne(q/q)   ) << endl;
  GlobalOutput() << " p > q          gives  " << (p > q        ) << endl;

  // We cannot write a + p because they belong to different rings.
  // See ex-RingHom*.C for how to "move" values from one ring to another.
}

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-ring1.C,v 1.2 2007/03/22 22:44:42 abbott Exp $
// $Log: ex-ring1.C,v $
// Revision 1.2  2007/03/22 22:44:42  abbott
// Removed spaces at ends of lines.
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.7  2007/03/07 18:09:19  bigatti
// -- first import
//
